#!/bin/bash

set -x

pflag=false
rflag=false
fflag=false

DIRNAME=$(pwd)

usage () { echo "
    -h -- Opens up this help message
    -p -- Name of the AWS profile to use
    -r -- AWS Region to deploy to (e.g. eu-west-1)
    -f -- Name of the parameter file to be used
"; }
options=':p:r:f:h'
while getopts $options option
do
    case "$option" in
        p  ) pflag=true; PROFILE=${OPTARG};;
        r  ) rflag=true; REGION=${OPTARG};;
        f  ) fflag=true; FILE=${OPTARG};;
        h  ) usage; exit;;
        \? ) echo "Unknown option: -$OPTARG" >&2; exit 1;;
        :  ) echo "Missing option argument for -$OPTARG" >&2; exit 1;;
        *  ) echo "Unimplemented option: -$OPTARG" >&2; exit 1;;
    esac
done

if ! $pflag
then
    echo "-p not specified, using default..." >&2
    PROFILE="default"
fi
if ! $rflag
then
    echo "-r not specified, using default region..." >&2
    REGION=$(aws configure get region --profile ${PROFILE})
fi
if ! $fflag
then
    echo "-f not specified, using default parameter file (parameters-multiple-values.json)..." >&2
    FILE="parameters-multiple-values.json"
fi

ACCOUNT_ID=$(aws sts get-caller-identity --query 'Account' --output text --profile ${PROFILE})
S3_BUCKET="cfn-artifacts-${ACCOUNT_ID}-${REGION}"

echo "Creating the S3 bucket for CloudFormation artifacts."

S3_EXISTS=$(aws s3 ls --region ${REGION} --profile ${PROFILE} | grep ${S3_BUCKET})
if [ -z "${S3_EXISTS// }" ]
then
    aws s3 mb s3://${S3_BUCKET} \
        --region ${REGION} \
        --profile ${PROFILE}
fi

mkdir $DIRNAME/output
aws cloudformation package \
    --template-file $DIRNAME/template.yaml \
    --s3-bucket $S3_BUCKET \
    --output-template-file $DIRNAME/output/packaged-template.yaml \
    --region ${REGION} \
    --profile ${PROFILE}

STACK_NAME=sns-endpoints-subscription
echo "Checking if stack exists ..."
if ! aws cloudformation describe-stacks --profile $PROFILE --region ${REGION} --stack-name $STACK_NAME; then
  echo -e "Stack does not exist, creating ..."
  aws cloudformation create-stack \
    --stack-name $STACK_NAME \
    --parameters file://$DIRNAME/${FILE} \
    --template-body file://$DIRNAME/output/packaged-template.yaml \
    --capabilities "CAPABILITY_NAMED_IAM" "CAPABILITY_AUTO_EXPAND" \
    --region ${REGION} \
    --profile $PROFILE

  echo "Waiting for stack to be created ..."
  aws cloudformation wait stack-create-complete \
    --profile $PROFILE \
    --region ${REGION} \
    --stack-name $STACK_NAME
else
  echo -e "Stack exists, attempting update ..."

  set +e
  update_output=$( aws cloudformation update-stack \
    --stack-name $STACK_NAME \
    --parameters file://$DIRNAME/${FILE} \
    --template-body file://$DIRNAME/output/packaged-template.yaml \
    --capabilities "CAPABILITY_NAMED_IAM" "CAPABILITY_AUTO_EXPAND" \
    --region ${REGION} \
    --profile $PROFILE 2>&1)
  status=$?
  set -e

  echo "$update_output"

  if [ $status -ne 0 ] ; then
    # Don't fail for no-op update
    if [[ $update_output == *"ValidationError"* && $update_output == *"No updates"* ]] ; then
      echo -e "\nFinished create/update - no updates to be performed";
      exit 0;
    else
      exit $status
    fi
  fi

  echo "Waiting for stack update to complete ..."
  aws cloudformation wait stack-update-complete \
    --region ${REGION} \
    --profile $PROFILE \
    --stack-name $STACK_NAME 
  echo "Finished create/update successfully!"
fi