import json
import logging
import traceback

import boto3

from urllib.request import Request, urlopen

logger = logging.getLogger()
logger.setLevel(logging.INFO)
logging.basicConfig(
    format='%(levelname)s %(threadName)s [%(filename)s:%(lineno)d] %(message)s',
    datefmt='%Y-%m-%d:%H:%M:%S',
    level=logging.INFO
)

try:
    logger.info("Container initialization completed")
except Exception as e:
    logger.error(e, exc_info=True)
    init_failed = e

def send_response(e, c, rs, rd):
    """ 
    Packages response and send signals to CloudFormation 
    :param e: The event given to this Lambda function 
    :param c: Context object, as above 
    :param rs: Returned status to be sent back to CFN 
    :param rd: Returned data to be sent back to CFN 
    """
    r = json.dumps({
        "Status": rs,
        "Reason": "CloudWatch Log Stream: " + c.log_stream_name,
        "PhysicalResourceId": e['LogicalResourceId'],
        "StackId": e['StackId'],
        "RequestId": e['RequestId'],
        "LogicalResourceId": e['LogicalResourceId'],
        "Data": rd
    })
    d = str.encode(r)
    h = {
        'content-type': '',
        'content-length': str(len(d))
    }
    req = Request(e['ResponseURL'], data=d, method='PUT', headers=h)
    r = urlopen(req)
    logger.info("Status message: {} {}".format(r.msg, r.getcode()))

def subscribe_endpoint(client, logger, topic_arn, endpoint, protocol):
    """ 
    :param event: Event sent from CFN when the custom resource is created 
    :param client: The Boto3 client for interactions with AWS API
    :param logger: Logger used for Lambda messages sent to CloudWatch
    :param topic_arn: Topic ARN for subscription
    :param endpoint: Endpoint to be subscribed to Topic ARN
    :param protocol: Protocol of the endpoint being subscribed
    """

    response_data = client.subscribe(
        TopicArn=topic_arn,
        Protocol=protocol,
        Endpoint=endpoint,
        ReturnSubscriptionArn=True
    )
    
    if response_data['ResponseMetadata']['HTTPStatusCode'] == 200: 
        logger.info(
            "Endpoint %s subscribed to topic %s.", endpoint, topic_arn)
    else:
        raise Exception(
            "Failed when subscribing %s to topic %s.", endpoint, topic_arn)
    
def adjust_subscriptions(event, client, logger):
    """ 
    :param event: Event sent from CFN when the custom resource is created 
    :param client: The Boto3 client for interactions with AWS API
    :param logger: Logger used for Lambda messages sent to CloudWatch
    """
    
    resource_properties        = event['ResourceProperties']
    topic_arn                  = resource_properties['TopicArn']
    subscription_protocol      = resource_properties['SubscriptionProtocol']
    subscription_endpoints     = resource_properties['SubscriptionEndpoints']
    old_resource_properties    = event['OldResourceProperties'] if 'OldResourceProperties' in event else None
    old_subscription_endpoints = old_resource_properties['SubscriptionEndpoints'] if old_resource_properties else None

    if subscription_endpoints:
        for endpoint in subscription_endpoints:
            if old_subscription_endpoints is None or endpoint not in old_subscription_endpoints:
                subscribe_endpoint(client, logger, topic_arn, endpoint, subscription_protocol)

def lambda_handler(event, context):
    """ 
    Entrypoint to Lambda, updates the main CloudTrail trail 
    :param event: The event given to this Lambda function 
    :param context: Context object containing Lambda metadata   
    """
    request_type = event['RequestType']
    sns_client = boto3.client('sns')

    print(event)
    print(context)

    try:
        if request_type == 'Create' or request_type == 'Update':
            adjust_subscriptions(event, sns_client, logger)
            
            message = None
            if request_type == 'Create':
                message = {
                    "Message": "Created"
                }
            elif request_type == 'Update':
                message = {
                    "Message": "Updated"
                }
                
            send_response(event, context, "SUCCESS", message)
            
        else:
            send_response(event, context, "SUCCESS", {"Message": "Function Not Applicable"})

    except Exception as ex:
        logger.error(ex)
        traceback.print_tb(ex.__traceback__)
        send_response(
            event,
            context,
            "FAILED",
            {
                "Message": "Exception"
            }
        )